@@ -4,10 +4,10 @@ from json import dumps
 import time
 
 from prawcore import Redirect
-from six.moves.urllib.parse import urljoin  # pylint: disable=import-error
 
-from ...const import API_PATH
-from ..util import stream_generator
+from ...const import API_PATH, urljoin
+from ...exceptions import APIException
+from ..util import permissions_string, stream_generator
 from ..listing.generator import ListingGenerator
 from ..listing.mixins import SubredditListingMixin
 from .base import RedditBase
@@ -62,7 +62,8 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
                           over_18=None, public_description=None,
                           public_traffic=None, show_media=None,
                           show_media_preview=None, spam_comments=None,
-                          spam_links=None, spam_selfposts=None, sr=None,
+                          spam_links=None, spam_selfposts=None,
+                          spoilers_enabled=None, sr=None,
                           submit_link_label=None, submit_text=None,
                           submit_text_label=None, subreddit_type=None,
                           suggested_comment_sort=None, title=None,
@@ -90,6 +91,7 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
                  'spam_comments': spam_comments,
                  'spam_links': spam_links,
                  'spam_selfposts': spam_selfposts,
+                 'spoilers_enabled': spoilers_enabled,
                  'sr': sr,
                  'submit_link_label': submit_link_label,
                  'submit_text': submit_text,
@@ -122,6 +124,13 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
 
            reddit.subreddit('SUBREDDIT').banned.add('NAME', reason='a reason')
 
+        To list the banned users along with any notes, try:
+
+        .. code-block:: python
+
+           for ban in reddit.subreddit('SUBREDDIT').banned():
+               print('{}: {}'.format(ban, ban.note))
+
         """
         if self._banned is None:
             self._banned = SubredditRelationship(self, 'banned')
@@ -168,7 +177,22 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
 
     @property
     def moderator(self):
-        """An instance of :class:`.ModeratorRelationship`."""
+        """An instance of :class:`.ModeratorRelationship`.
+
+        For example to add a moderator try:
+
+        .. code-block:: python
+
+           reddit.subreddit('SUBREDDIT').moderator.add('NAME')
+
+        To list the moderators along with their permissions try:
+
+        .. code-block:: python
+
+           for moderator in reddit.subreddit('SUBREDDIT').moderator():
+               print('{}: {}'.format(moderator, moderator.mod_permissions))
+
+        """
         if self._moderator is None:
             self._moderator = ModeratorRelationship(self, 'moderator')
         return self._moderator
@@ -184,7 +208,7 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
     def quaran(self):
         """An instance of :class:`.SubredditQuarantine`.
 
-        This property is named ``quaran`` because ``quarantine`` is an
+        This property is named ``quaran`` because ``quarantine`` is a
         Subreddit attribute returned by Reddit to indicate whether or not a
         Subreddit is quarantined.
 
@@ -195,7 +219,26 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
 
     @property
     def stream(self):
-        """An instance of :class:`.SubredditStream`."""
+        """An instance of :class:`.SubredditStream`.
+
+        Streams can be used to indefinitely retrieve new comments made to a
+        subreddit, like:
+
+        .. code:: python
+
+           for comment in reddit.subreddit('iama').stream.comments():
+               print(comment)
+
+        Additionally, new submissions can be retrieved via the stream. In the
+        following example all submissions are fetched via the special subreddit
+        ``all``:
+
+        .. code:: python
+
+           for submission in reddit.subreddit('all').stream.submissions():
+               print(submission)
+
+        """
         if self._stream is None:
             self._stream = SubredditStream(self)
         return self._stream
@@ -209,7 +252,23 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
 
     @property
     def wiki(self):
-        """An instance of :class:`.SubredditWiki`."""
+        """An instance of :class:`.SubredditWiki`.
+
+        This attribute can be used to discover all wikipages for a subreddit:
+
+        .. code:: python
+
+           for wikipage in reddit.subreddit('iama').wiki:
+               print(wikipage)
+
+        To fetch the content for a given wikipage try:
+
+        .. code:: python
+
+           wikipage = reddit.subreddit('iama').wiki['proof']
+           print(wikipage.content_md)
+
+        """
         if self._wiki is None:
             self._wiki = SubredditWiki(self)
         return self._wiki
@@ -220,7 +279,7 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
         :param reddit: An instance of :class:`~.Reddit`.
         :param display_name: The name of the subreddit.
 
-        .. note:: This class should not be initialized directly instead obtain
+        .. note:: This class should not be initialized directly. Instead obtain
            an instance via: ``reddit.subreddit('subreddit_name')``
 
         """
@@ -248,6 +307,10 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
         return self._submission_class(self._reddit, url=urljoin(
             self._reddit.config.reddit_url, path))
 
+    def rules(self):
+        """Return rules for the subreddit."""
+        return self._reddit.get(API_PATH['rules'].format(subreddit=self))
+
     def search(self, query, sort='relevance', syntax='cloudsearch',
                time_filter='all', **generator_kwargs):
         """Return a ListingGenerator for items that match ``query``.
@@ -271,7 +334,7 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
                print(submission.title)
 
         """
-        self.validate_time_filter(time_filter)
+        self._validate_time_filter(time_filter)
         not_all = self.display_name.lower() != 'all'
         self._safely_add_arguments(generator_kwargs, 'params', q=query,
                                    restrict_sr=not_all, sort=sort,
@@ -283,7 +346,7 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
         """Return a Submission object for a sticky of the subreddit.
 
         :param number: Specify which sticky to return. 1 appears at the top
-            (Default: 1).
+            (default: 1).
 
         Raises ``prawcore.NotFound`` if the sticky does not exist.
 
@@ -301,13 +364,13 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
 
         :param start: A UNIX timestamp indicating the earliest creation time of
             submission yielded during the call. A value of ``None`` will
-            consider all submissions older than ``end`` (Default: None).
+            consider all submissions older than ``end`` (default: None).
         :param end: A UNIX timestamp indicating the latest creation time of a
             submission yielded during the call. A value of ``None`` will
-            consider all submissions newer than ``start`` (Default: None).
+            consider all submissions newer than ``start`` (default: None).
         :param extra_query: A cloudsearch query that will be combined via
             ``(and timestamp:start..end EXTRA_QUERY)`` to futher filter
-            results (Default: None).
+            results (default: None).
 
         Submissions are yielded newest first.
 
@@ -360,9 +423,9 @@ class Subreddit(RedditBase, MessageableMixin, SubredditListingMixin):
             submission.
         :param url: The URL for a ``link`` submission.
         :param resubmit: When False, an error will occur if the URL has already
-            been submitted (Default: True).
+            been submitted (default: True).
         :param send_replies: When True, messages will be sent to the submission
-            author when comments are made to the submission (Default: True).
+            author when comments are made to the submission (default: True).
         :returns: A :class:`~.Submission` object for the newly created
             submission.
 
@@ -506,8 +569,8 @@ class SubredditFlair(object):
     def __call__(self, redditor=None, **generator_kwargs):
         """Return a generator for Redditors and their associated flair.
 
-        :param redditor: Yield at most a single :class:`~.Redditor`
-            instance.
+        :param redditor: When provided, yield at most a single
+            :class:`~.Redditor` instance (default: None).
 
         This method is intended to be used like:
 
@@ -519,6 +582,7 @@ class SubredditFlair(object):
         """
         Subreddit._safely_add_arguments(generator_kwargs, 'params',
                                         name=redditor)
+        generator_kwargs.setdefault('limit', None)
         url = API_PATH['flairlist'].format(subreddit=self.subreddit)
         return ListingGenerator(self.subreddit._reddit, url,
                                 **generator_kwargs)
@@ -576,14 +640,15 @@ class SubredditFlair(object):
     def delete(self, redditor):
         """Delete flair for a Redditor.
 
-        :param redditor: An instance of :class:`.Redditor`, or the name of a
-            Redditor.
+        :param redditor: A redditor name (e.g., ``'spez'``) or
+            :class:`~.Redditor` instance.
 
         .. note:: To delete the flair of many Redditors at once, please see
                   :meth:`~praw.models.reddit.subreddit.SubredditFlair.update`.
 
         """
         url = API_PATH['deleteflair'].format(subreddit=self.subreddit)
+        # PRAW5 REMOVE (return statement)
         return self.subreddit._reddit.post(url, data={'name': str(redditor)})
 
     def delete_all(self):
@@ -593,17 +658,17 @@ class SubredditFlair(object):
             each delete.
 
         """
-        return self.update(x['user'] for x in self)
+        return self.update(x['user'] for x in self())
 
     def set(self, redditor=None, text='', css_class='', thing=None):
         """Set flair for a Redditor.
 
-        :param redditor: (Required) An instance of Redditor or a string that is
-            the name of a Redditor.
+        :param redditor: (Required) A redditor name (e.g., ``'spez'``) or
+            :class:`~.Redditor` instance.
         :param text: The flair text to associate with the Redditor or
-            Submission (Default: '').
+            Submission (default: '').
         :param css_class: The css class to associate with the flair html
-            (Default: '').
+            (default: '').
 
         This method can only be used by an authenticated user who is a
         moderator of the associated Subreddit.
@@ -639,20 +704,30 @@ class SubredditFlair(object):
     def update(self, flair_list, text='', css_class=''):
         """Set or clear the flair for many Redditors at once.
 
-        :param redditor_flair_list: Each item in this list should be either: a
-            Redditor name string, a Redditor, or a dictionary containing the
-            keys ``user``, ``flair_text``, and ``flair_css_class``. The
-            ``user`` key should map to a Redditor name string, or a
-            Redditor. When a dictionary isn't provided, or the dictionary is
-            missing one of ``flair_text``, or ``flair_css_class`` attributes
-            the default values will come from the the following arguments.
+        :param flair_list: Each item in this list should be either: the name of
+            a Redditor, an instance of :class:`.Redditor`, or a dictionary
+            mapping keys ``user``, ``flair_text``, and ``flair_css_class`` to
+            their respective values. The ``user`` key should map to a Redditor,
+            as described above. When a dictionary isn't provided, or the
+            dictionary is missing one of ``flair_text``, or ``flair_css_class``
+            attributes the default values will come from the the following
+            arguments.
+
         :param text: The flair text to use when not explicitly provided in
-            ``flair_list`` (Default: '').
+            ``flair_list`` (default: '').
         :param css_class: The css class to use when not explicitly provided in
-            ``flair_list`` (Default: '').
+            ``flair_list`` (default: '').
         :returns: List of dictionaries indicating the success or failure of
             each update.
 
+        For example to clear the flair text, and set the `praw` flair css class
+        on a few users try:
+
+        .. code:: python
+
+           subreddit.flair.update(['bboe', 'spez', 'spladug'],
+                                  css_class='praw')
+
         """
         lines = []
         for item in flair_list:
@@ -988,6 +1063,7 @@ class SubredditModeration(object):
             One of ``all``, ``low``, ``high``.
         :param spam_selfposts: Spam filter strength for selfposts.
             One of ``all``, ``low``, ``high``.
+        :param spoilers_enabled: Enable marking posts as containing spoilers.
         :param sr: The fullname of the subreddit whose settings will be
             updated.
         :param submit_link_label: Custom label for submit link button
@@ -1012,7 +1088,15 @@ class SubredditModeration(object):
         Additional keyword arguments can be provided to handle new settings as
         Reddit introduces them.
 
-        Unspecified settings will maintain their current value.
+        Settings that are documented here and aren't explitly set by you in a
+        call to :meth:`.SubredditModeration.update` should retain their current
+        value. If they do not please file a bug.
+
+        .. warning:: Undocumented settings, or settings that were very recently
+                     documented, may not retain their current value when
+                     updating. This often occurs when Reddit adds a new setting
+                     but forgets to add that setting to the API endpoint that
+                     is used to fetch the current settings.
 
         """
         current_settings = self.settings()
@@ -1102,9 +1186,10 @@ class SubredditRelationship(object):
     def __call__(self, redditor=None, **generator_kwargs):
         """Return a generator for Redditors belonging to this relationship.
 
-        :param redditor: Yield at most a single :class:`~.Redditor`
-            instance. This is useful to confirm if a relationship exists, or to
-            fetch the metadata associated with a particular relationship.
+        :param redditor: When provided, yield at most a single
+            :class:`~.Redditor` instance. This is useful to confirm if a
+            relationship exists, or to fetch the metadata associated with a
+            particular relationship (default: None).
 
         Additional keyword arguments are passed in the initialization of
         :class:`.ListingGenerator`.
@@ -1144,7 +1229,8 @@ class SubredditRelationship(object):
     def add(self, redditor, **other_settings):
         """Add ``redditor`` to this relationship.
 
-        :param redditor: A string or :class:`~.Redditor` instance.
+        :param redditor: A redditor name (e.g., ``'spez'``) or
+            :class:`~.Redditor` instance.
 
         """
         data = {'name': str(redditor), 'type': self.relationship}
@@ -1155,7 +1241,8 @@ class SubredditRelationship(object):
     def remove(self, redditor):
         """Remove ``redditor`` from this relationship.
 
-        :param redditor: A string or :class:`~.Redditor` instance.
+        :param redditor: A redditor name (e.g., ``'spez'``) or
+            :class:`~.Redditor` instance.
 
         """
         data = {'name': str(redditor), 'type': self.relationship}
@@ -1170,7 +1257,7 @@ class ContributorRelationship(SubredditRelationship):
 
     .. code-block:: python
 
-       for contributor in reddit.subreddit('redditdev').contributors():
+       for contributor in reddit.subreddit('redditdev').contributor():
            print(contributor)
 
     """
@@ -1188,30 +1275,57 @@ class ModeratorRelationship(SubredditRelationship):
 
     .. code-block:: python
 
-       for moderator in reddit.subreddit('redditdev').moderators():
+       for moderator in reddit.subreddit('redditdev').moderator():
            print(moderator)
 
     """
 
+    PERMISSIONS = {'access', 'config', 'flair', 'mail', 'posts', 'wiki'}
+
     @staticmethod
     def _handle_permissions(permissions, other_settings):
-        if isinstance(permissions, list):
-            other_settings = deepcopy(other_settings) if other_settings else {}
-            if permissions:
-                permissions = ['+{}'.format(x) for x in permissions]
-            else:
-                # A single permission prefixed with `-` must be provided in
-                # order to have no permissions set. `-all` unfortunately is
-                # treated as if no permissions are passed thus resulting in
-                # full permissions.
-                permissions = ['-access']
-            other_settings['permissions'] = ','.join(permissions)
+        other_settings = deepcopy(other_settings) if other_settings else {}
+        other_settings['permissions'] = permissions_string(
+            permissions, ModeratorRelationship.PERMISSIONS)
         return other_settings
 
+    def __call__(self, redditor=None):
+        """Return a list of Redditors who are moderators.
+
+        :param redditor: When provided, return a list containing at most one
+            :class:`~.Redditor` instance. This is useful to confirm if a
+            relationship exists, or to fetch the metadata associated with a
+            particular relationship (default: None).
+
+        .. note:: Unlike other relationship callables, this relationship is not
+                  paginated. Thus it simply returns the full list, rather than
+                  an iterator for the results.
+
+        To be used like:
+
+        .. code:: python
+
+           moderators = reddit.subreddit('nameofsub').moderator()
+
+        For example, to list the moderators along with their permissions try:
+
+        .. code:: python
+
+           for moderator in reddit.subreddit('SUBREDDIT').moderator():
+               print('{}: {}'.format(moderator, moderator.mod_permissions))
+
+
+        """
+        params = {} if redditor is None else {'user': redditor}
+        url = API_PATH['list_{}'.format(self.relationship)].format(
+            subreddit=self.subreddit)
+        return self.subreddit._reddit.get(url, params=params)
+
     def add(self, redditor, permissions=None, **other_settings):
         """Add or invite ``redditor`` to be a moderator of the subreddit.
 
-        :param redditor: A string or :class:`~.Redditor` instance.
+        :param redditor: A redditor name (e.g., ``'spez'``) or
+            :class:`~.Redditor` instance.
         :param permissions: When provided (not `None`), permissions should be a
             list of strings specifying which subset of permissions to grant. An
             empty list `[]` indicates no permissions, and when not provided
@@ -1227,7 +1341,8 @@ class ModeratorRelationship(SubredditRelationship):
     def invite(self, redditor, permissions=None, **other_settings):
         """Invite ``redditor`` to be a moderator of the subreddit.
 
-        :param redditor: A string or :class:`~.Redditor` instance.
+        :param redditor: A redditor name (e.g., ``'spez'``) or
+            :class:`~.Redditor` instance.
         :param permissions: When provided (not `None`), permissions should be a
             list of strings specifying which subset of permissions to grant. An
             empty list `[]` indicates no permissions, and when not provided
@@ -1244,15 +1359,45 @@ class ModeratorRelationship(SubredditRelationship):
         self.subreddit._reddit.post(API_PATH['leavemoderator'],
                                     data={'id': self.subreddit.fullname})
 
+    def remove_invite(self, redditor):
+        """Remove the moderator invite for ``redditor``.
+
+        :param redditor: A redditor name (e.g., ``'spez'``) or
+            :class:`~.Redditor` instance.
+
+        Example:
+
+        .. code:: python
+
+           reddit.subreddit('subredditname').mod.remove_invite('spez')
+
+        """
+        data = {'name': str(redditor), 'type': 'moderator_invite'}
+        url = API_PATH['unfriend'].format(subreddit=self.subreddit)
+        self.subreddit._reddit.post(url, data=data)
+
     def update(self, redditor, permissions=None):
         """Update the moderator permissions for ``redditor``.
 
-        :param redditor: A string or :class:`~.Redditor` instance.
+        :param redditor: A redditor name (e.g., ``'spez'``) or
+            :class:`~.Redditor` instance.
         :param permissions: When provided (not `None`), permissions should be a
             list of strings specifying which subset of permissions to grant. An
-            empty list `[]` indicates no permissions, and when not provided
+            empty list `[]` indicates no permissions, and when not provided,
             `None`, indicates full permissions.
 
+        For example, to add all permissions to the moderator, try:
+
+        .. code:: python
+
+           subreddit.moderator.update('spez')
+
+        To remove all permissions from the moderator, try:
+
+        .. code:: python
+
+           subreddit.moderator.update('spez', [])
+
         """
         url = API_PATH['setpermissions'].format(subreddit=self.subreddit)
         data = self._handle_permissions(
@@ -1262,12 +1407,20 @@ class ModeratorRelationship(SubredditRelationship):
     def update_invite(self, redditor, permissions=None):
         """Update the moderator invite permissions for ``redditor``.
 
-        :param redditor: A string or :class:`~.Redditor` instance.
+        :param redditor: A redditor name (e.g., ``'spez'``) or
+            :class:`~.Redditor` instance.
         :param permissions: When provided (not `None`), permissions should be a
             list of strings specifying which subset of permissions to grant. An
-            empty list `[]` indicates no permissions, and when not provided
+            empty list `[]` indicates no permissions, and when not provided,
             `None`, indicates full permissions.
 
+        For example, to grant the flair and mail permissions to the moderator
+        invite, try:
+
+        .. code:: python
+
+           subreddit.moderator.update_invite('spez', ['flair', 'mail'])
+
         """
         url = API_PATH['setpermissions'].format(subreddit=self.subreddit)
         data = self._handle_permissions(
@@ -1292,6 +1445,14 @@ class SubredditStream(object):
         Comments are yielded oldest first. Up to 100 historical comments will
         initially be returned.
 
+        For example, to retrieve all new comments made to the ``iama``
+        subreddit, try:
+
+        .. code:: python
+
+           for comment in reddit.subreddit('iama').stream.comments():
+               print(comment)
+
         """
         return stream_generator(self.subreddit.comments)
 
@@ -1301,6 +1462,13 @@ class SubredditStream(object):
         Submissions are yielded oldest first. Up to 100 historical submissions
         will initially be returned.
 
+        For example to retrieve all new submissions made to all of Reddit, try:
+
+        .. code:: python
+
+           for submission in reddit.subreddit('all').stream.submissions():
+               print(submission)
+
         """
         return stream_generator(self.subreddit.new)
 
@@ -1311,7 +1479,15 @@ class SubredditStylesheet(object):
     JPEG_HEADER = b'\xff\xd8\xff'
 
     def __call__(self):
-        """Return the subreddit's stylesheet."""
+        """Return the subreddit's stylesheet.
+
+        To be used as:
+
+        .. code:: python
+
+           stylesheet = reddit.subreddit('SUBREDDIT').stylesheet()
+
+        """
         url = API_PATH['about_stylesheet'].format(subreddit=self.subreddit)
         return self.subreddit._reddit.get(url)
 
@@ -1320,6 +1496,12 @@ class SubredditStylesheet(object):
 
         :param subreddit: The subreddit associated with the stylesheet.
 
+        An instance of this class is provided as:
+
+        .. code:: python
+
+           reddit.subreddit('SUBREDDIT').stylesheet
+
         """
         self.subreddit = subreddit
 
@@ -1329,14 +1511,25 @@ class SubredditStylesheet(object):
             image.seek(0)
             data['img_type'] = 'jpg' if header == self.JPEG_HEADER else 'png'
             url = API_PATH['upload_image'].format(subreddit=self.subreddit)
-            return self.subreddit._reddit.post(
-                url, data=data, files={'file': image})
+            response = self.subreddit._reddit.post(url, data=data,
+                                                   files={'file': image})
+            if response['errors']:
+                assert response['errors'] == ['IMAGE_ERROR'], \
+                                             'Please file a bug with PRAW'
+                raise APIException(response['errors'][0], '', None)
+            return response
 
     def delete_header(self):
         """Remove the current subreddit header image.
 
         Succeeds even if there is no header image.
 
+        Example:
+
+        .. code:: python
+
+           reddit.subreddit('SUBREDDIT').stylesheet.delete_header()
+
         """
         url = API_PATH['delete_sr_header'].format(subreddit=self.subreddit)
         self.subreddit._reddit.post(url)
@@ -1346,15 +1539,58 @@ class SubredditStylesheet(object):
 
         Succeeds even if the named image does not exist.
 
+        Example:
+
+        .. code:: python
+
+           reddit.subreddit('SUBREDDIT').stylesheet.delete_image('smile')
+
         """
         url = API_PATH['delete_sr_image'].format(subreddit=self.subreddit)
         self.subreddit._reddit.post(url, data={'img_name': name})
 
+    def delete_mobile_header(self):
+        """Remove the current subreddit mobile header.
+
+        Succeeds even if there is no mobile header.
+
+        Example:
+
+        .. code:: python
+
+           reddit.subreddit('SUBREDDIT').stylesheet.delete_mobile_header()
+
+        """
+        url = API_PATH['delete_sr_header'].format(subreddit=self.subreddit)
+        self.subreddit._reddit.post(url)
+
+    def delete_mobile_icon(self):
+        """Remove the current subreddit mobile icon.
+
+        Succeeds even if there is no mobile icon.
+
+        Example:
+
+        .. code:: python
+
+           reddit.subreddit('SUBREDDIT').stylesheet.delete_ombile_icon()
+
+        """
+        url = API_PATH['delete_sr_icon'].format(subreddit=self.subreddit)
+        self.subreddit._reddit.post(url)
+
     def update(self, stylesheet, reason=None):
         """Update the subreddit's stylesheet.
 
         :param stylesheet: The CSS for the new stylesheet.
 
+        Example:
+
+        .. code:: python
+
+           reddit.subreddit('SUBREDDIT').stylesheet.update(
+               'p { color: green; }', 'color text green')
+
         """
         data = {'op': 'save', 'reason': reason,
                 'stylesheet_contents': stylesheet}
@@ -1370,6 +1606,19 @@ class SubredditStylesheet(object):
         :returns: A dictionary containing a link to the uploaded image under
             the key ``img_src``.
 
+        Raises ``prawcore.TooLarge`` if the overall request body is too large.
+
+        Raises :class:`.APIException` if there are other issues with the
+        uploaded image. Unfortunately the exception info might not be very
+        specific, so try through the website with the same image to see what
+        the problem actually might be.
+
+        Example:
+
+        .. code:: python
+
+           reddit.subreddit('SUBREDDIT').stylesheet.upload('smile', 'img.png')
+
         """
         return self._upload_image(image_path,
                                   {'name': name, 'upload_type': 'img'})
@@ -1381,15 +1630,85 @@ class SubredditStylesheet(object):
         :returns: A dictionary containing a link to the uploaded image under
             the key ``img_src``.
 
+        Raises ``prawcore.TooLarge`` if the overall request body is too large.
+
+        Raises :class:`.APIException` if there are other issues with the
+        uploaded image. Unfortunately the exception info might not be very
+        specific, so try through the website with the same image to see what
+        the problem actually might be.
+
+        Example:
+
+        .. code:: python
+
+           reddit.subreddit('SUBREDDIT').stylesheet.upload_header('header.png')
+
         """
         return self._upload_image(image_path, {'upload_type': 'header'})
 
+    def upload_mobile_header(self, image_path):
+        """Upload an image to be used as the Subreddit's mobile header.
+
+        :param image_path: A path to a jpeg or png image.
+        :returns: A dictionary containing a link to the uploaded image under
+            the key ``img_src``.
+
+        Raises ``prawcore.TooLarge`` if the overall request body is too large.
+
+        Raises :class:`.APIException` if there are other issues with the
+        uploaded image. Unfortunately the exception info might not be very
+        specific, so try through the website with the same image to see what
+        the problem actually might be.
+
+        For example:
+
+        .. code:: python
+
+           reddit.subreddit('SUBREDDIT').stylesheet.upload_mobile_header(
+               'header.png')
+
+        """
+        return self._upload_image(image_path, {'upload_type': 'banner'})
+
+    def upload_mobile_icon(self, image_path):
+        """Upload an image to be used as the Subreddit's mobile icon.
+
+        :param image_path: A path to a jpeg or png image.
+        :returns: A dictionary containing a link to the uploaded image under
+            the key ``img_src``.
+
+        Raises ``prawcore.TooLarge`` if the overall request body is too large.
+
+        Raises :class:`.APIException` if there are other issues with the
+        uploaded image. Unfortunately the exception info might not be very
+        specific, so try through the website with the same image to see what
+        the problem actually might be.
+
+        For example:
+
+        .. code:: python
+
+           reddit.subreddit('SUBREDDIT').stylesheet.upload_mobile_icon(
+               'icon.png')
+
+        """
+        return self._upload_image(image_path, {'upload_type': 'icon'})
+
 
 class SubredditWiki(object):
     """Provides a set of moderation functions to a Subreddit."""
 
     def __getitem__(self, page_name):
-        """Lazily return the WikiPage for the subreddit named ``page_name``."""
+        """Lazily return the WikiPage for the subreddit named ``page_name``.
+
+        This method is to be used to fetch a specific wikipage, like so:
+
+        .. code:: python
+
+           wikipage = reddit.subreddit('iama').wiki['proof']
+           print(wikipage.content_md)
+
+        """
         return WikiPage(self.subreddit._reddit, self.subreddit,
                         page_name.lower())
 
@@ -1400,11 +1719,20 @@ class SubredditWiki(object):
 
         """
         self.banned = SubredditRelationship(subreddit, 'wikibanned')
-        self.contributors = SubredditRelationship(subreddit, 'wikicontributor')
+        self.contributor = SubredditRelationship(subreddit, 'wikicontributor')
         self.subreddit = subreddit
 
     def __iter__(self):
-        """Iterate through the pages of the wiki."""
+        """Iterate through the pages of the wiki.
+
+        This method is to be used to discover all wikipages for a subreddit:
+
+        .. code:: python
+
+           for wikipage in reddit.subreddit('iama').wiki:
+               print(wikipage)
+
+        """
         response = self.subreddit._reddit.get(
             API_PATH['wiki_pages'].format(subreddit=self.subreddit),
             params={'unique': self.subreddit._reddit._next_unique})
